<?php
class powersocket {
	
	//* Setting Properties *//
	private $ip; 			//IP address of powersocket
	private $pw; 			//Password for powersocket webinterface

	//* Helper Properties *//
	private $hasLogin = false;	//login state
	private $sockets = Array( 	//status for each socket (1-4) of powersocket
		1 => null,
		2 => null,
		3 => null,
		4 => null
	);
	private $error;			//error messages
	private $debug = "";		//debug log
	
    /**
     * Working Constructor
     * .. maybe with default values?
     */
    public function __construct() {
        $this->ip = "http://192.168.42.201/";
        $this->pw = "Power";
    }
    
    /**
     * Destructor
     * logout from webinterface.
     */
    public function __destruct() { $this->socket_logout(); }
    
    /**
     * set Remote IP of powersocket
     */
    public function setIp($newip) {
    	$this->ip = $newip;
    }
    /**
     * set Password for powersocket webinterface
     */
    public function setPassword($newpassword) {
    	$this->pw = $newpassword;
    }
    
    	/**
     	* login into powersocket
	* establish a session (by IP of sender)
     	*/
	private function socket_login() {
		//already login? skip.
		if ($this->hasLogin)
			return true;
		
		$c = curl_init();
		curl_setopt($c, CURLOPT_URL, $this->ip . "login.html");
		curl_setopt($c, CURLOPT_CONNECTTIMEOUT, 5);
		curl_setopt($c, CURLOPT_POST, true);
		curl_setopt($c, CURLOPT_POSTFIELDS, "pw=".$this->pw);
		curl_setopt($c, CURLOPT_RETURNTRANSFER, true);
		$curlresult = curl_exec($c);
		curl_close($c); 
		
		if (empty($curlresult)) {
			$this->logDebug("Login failed, no curl result.");
			return false;
		}
		else {
			$this->hasLogin = true;
			return true;
		}
	}
	
    	/**
     	* logout of powersocket webinterface
	* close session (by IP of sender)
     	*/
	private function socket_logout() {
		
		$c = curl_init();
		curl_setopt($c, CURLOPT_URL, $this->ip . "login.html");
		curl_setopt($c, CURLOPT_CONNECTTIMEOUT, 5);
		curl_setopt($c, CURLOPT_RETURNTRANSFER, true);
		$curlresult = curl_exec($c);
		curl_close($c); 
		
		$this->hasLogin = false;
		return true;
	}
	
	/*
	 * Get States of all 4 sockets, returns Socket Array.
	 *   login(), request, logout().
	 * @return: Array or false
	 */
	public function getSocketStates() {
		$login = $this->socket_login();
		if (!$login)
			return false;
		
		$c = curl_init();
		curl_setopt($c, CURLOPT_URL, $this->ip . "status.html");
		curl_setopt($c, CURLOPT_CONNECTTIMEOUT, 5);
		curl_setopt($c, CURLOPT_RETURNTRANSFER, true);
		$curlresult = curl_exec($c);
		curl_close($c); 
		
		//Get States
	
		$ctlpos = strpos($curlresult, "var ctl = [");
		if ($ctlpos) {
			$socketctls = substr($curlresult, $ctlpos+11, 7);
			
			//override private array
			$this->sockets[1] = substr($socketctls, 0,1);
			$this->sockets[2] = substr($socketctls, 2,1);
			$this->sockets[3] = substr($socketctls, 4,1);
			$this->sockets[4] = substr($socketctls, 6,1);
			
			$this->logDebug('Socket CTLs: ' . $this->sockets[1] .$this->sockets[2] .$this->sockets[3] .$this->sockets[4]);
			
			for ($i=1; $i<=4; $i++) {
				if ($this->sockets[$i] == "1") $this->sockets[$i] = true;
				else if ($this->sockets[$i] == "0") $this->sockets[$i] = false;
			}
			
			return $this->sockets;
		} else { 
			$this->logDebug("Socket CTLs not found<br/>");
			return false;
		}
	}
	
	/*
	 * Switch specified socket into new state
	 * @param $socket	1-4
	 * @param $state	true (on) or false (off)
	 * @return: true or false
	 */
	public function switchSocket($socket, $state) {
		//login
		$login = $this->socket_login();
		//catch no login
		if (!$login)
			return false;
		
		$oldstates = $this->getSocketStates();
		if ($oldstates[$socket] == $state) {
			$this->logDebug('Switch Socket is already in target State. Exit with true.');
			return true;
		}
		
		$newstate = $state ? 1 : 0;
		
		$socketuri = $this->ip; // . "status.html?sn=" . $socket;
		$switchstate = "ctl".$socket."=".$newstate;
		
		$this->debug .= "socketuri: " . $socketuri . "<br/>";
		$this->debug .= "switchstate: " . $switchstate . "<br/>";
		
		$c = curl_init();
		curl_setopt($c, CURLOPT_URL, $socketuri);
		curl_setopt($c, CURLOPT_CONNECTTIMEOUT, 10);
		curl_setopt($c, CURLOPT_POST, true);
		curl_setopt($c, CURLOPT_POSTFIELDS, $switchstate);
		curl_setopt($c, CURLOPT_RETURNTRANSFER, true);
		$curlresult = curl_exec($c);
		
		curl_close($c); 
		
		//Check if socket has changed to new state. Return result.
		$newstates = $this->getSocketStates();
		if ($newstates[$socket] == $state) {
			return true;
		} else {
			$this->logDebug('Switch Socket failed. No change of socket state.');
			return false;
		}
	}
	
	public function getSocketStates_json() {
		return json_encode($this->getSocketStates());
	}
	
	public function getDebug() {
		return "<br/>" . $this->debug;
	}
	private function logDebug($newdebug) {
		$this->debug .= $newdebug . "<br/>";
	}
}
?>
