package edu.dhbw.brainbox.outputs;

import java.io.FileInputStream;
import java.io.IOException;
import java.util.List;
import java.util.Properties;

import org.apache.http.HttpEntity;
import org.apache.http.HttpException;
import org.apache.http.HttpHost;
import org.apache.http.HttpResponse;
import org.apache.http.HttpStatus;
import org.apache.http.client.HttpClient;
import org.apache.http.client.methods.HttpGet;
import org.apache.http.conn.params.ConnRoutePNames;
import org.apache.http.impl.client.DefaultHttpClient;
import org.apache.http.params.HttpConnectionParams;
import org.apache.http.params.HttpParams;
import org.apache.http.util.EntityUtils;

public class AktorHTTP extends BaseAktor {
	private String url;
	private String data;
	protected Boolean proxy = false;
	protected String proxyhost;
	protected Integer proxyport;
	
	//Standard Getters & Setters	
	public String getUrl() {
		return url;
	}

	public void setUrl(String url) {
		this.url = url;
	}

	public String getData() {
		return data;
	}

	public void setData(String data) {
		this.data = data;
	}

	//Working Constructor
	public AktorHTTP(String id, String url, String data) {
		super(id);
		this.url = url;
		this.data = data;
		
		//Proxy by Props
		Properties prop = new Properties();
        try {
                prop.load(new FileInputStream("brainbox.properties"));
                
                proxy = Boolean.parseBoolean(prop.getProperty("proxy"));
                proxyhost = prop.getProperty("proxyhost");
                proxyport = Integer.parseInt(prop.getProperty("proxyport"));
                
                System.out.println("Proxyhost = " + proxyhost);
                System.out.println("proxyport = " + proxyport);
        } catch (IOException e) {
                System.err.println( "Settings property file not found. (Required for specify a port).\n");
        }
	}

	//Functional
	public Boolean send() {
		return send(this.url, this.data);
	}
	
	private Boolean send(String url, String data) {
		HttpClient httpclient = new DefaultHttpClient();
		HttpParams params = httpclient.getParams();
		HttpConnectionParams.setConnectionTimeout(params, 5000);
		HttpConnectionParams.setSoTimeout(params, 10000);
		
		if (this.proxy && this.proxyhost != null && this.proxyport != null) {
			HttpHost proxy = new HttpHost(proxyhost, proxyport);
			httpclient.getParams().setParameter(ConnRoutePNames.DEFAULT_PROXY, proxy);
		}
		
		try {
		    // Send data
		    HttpGet httpget = new HttpGet(url+"?"+data);

            HttpResponse response = httpclient.execute(httpget);
            HttpEntity entity = response.getEntity();

            if (response.getStatusLine().getStatusCode() != HttpStatus.SC_OK) {
            	throw new HttpException();
            }

            //Check if login response content contains socket states
            if (entity != null) {
            	String returnhtml =  EntityUtils.toString(entity);
            	//TODO: check http status code
            	if (returnhtml != null) {
            		System.out.println("[Debug] aktorHTTP Returned: " + returnhtml);
            		return true;
            	}
            	else
            		return false;
            }
            EntityUtils.consume(entity);
           
		} catch (Exception e) {
			System.out.println("Exception while sending HTTP");
			System.err.println(e.getMessage());
			e.printStackTrace();
		} finally {
			httpclient.getConnectionManager().shutdown();
		}
		
		return false;
	}
	
	public Boolean fireAction(List<String> actions) {
		if (actions.size() == 2) {
			try {
				send(actions.get(0), actions.get(1));
				return true;
			} catch (Exception e) {
				System.err.println("Parse Error to switch Socket in ActionString: "+e.getLocalizedMessage());
				e.printStackTrace();
				return false;
			}
		} else {
			System.err.println("Invalid ActionString to fire Action. Should length = 2.");
			return false;
		}
	}
	
	public String toString() {
		return "AktorHTTP #"+getId()+" url: '" + this.url + "', data: " + this.data.substring(0, 25).concat("...");
	}
	
	public String toJSON() {
		StringBuilder sb = new StringBuilder();
        sb.append("{\n");
        sb.append("\"id\": \"" + getId() + "\",\n");
        sb.append("\"url\": \""+ this.url + "\",\n");
        sb.append("\"data\": \""+ this.data + "\",\n");
        sb.append("}");
        return sb.toString();
	}
	
}
