package edu.dhbw.brainbox.outputs;
import java.io.IOException;
import java.net.SocketException;
import java.util.ArrayList;
import java.util.List;

import org.apache.http.HttpEntity;
import org.apache.http.HttpException;
import org.apache.http.HttpResponse;
import org.apache.http.HttpStatus;
import org.apache.http.NameValuePair;
import org.apache.http.client.ClientProtocolException;
import org.apache.http.client.HttpClient;
import org.apache.http.client.entity.UrlEncodedFormEntity;
import org.apache.http.client.methods.HttpGet;
import org.apache.http.client.methods.HttpPost;
import org.apache.http.impl.client.DefaultHttpClient;
import org.apache.http.message.BasicNameValuePair;
import org.apache.http.params.HttpConnectionParams;
import org.apache.http.params.HttpParams;
import org.apache.http.protocol.HTTP;
import org.apache.http.util.EntityUtils;

import edu.dhbw.brainbox.exceptions.PowersocketUnreachableException;


public class Powersocket extends BaseAktor {
	private String powerid;
	private String ip;
	private String password;
	private String comment;

	private Boolean[] socketStates = {null, null, null, null};

	//Standard Getters & Setters
	public String getPowerid() {
		return powerid;
	}
	public void setPowerid(String powerid) {
		this.powerid = powerid;
	}
	public String getIp() {
		return ip;
	}

	public void setIp(String ip) {
		this.ip = ip;
	}

	public String getPassword() {
		return password;
	}

	public void setPassword(String password) {
		this.password = password;
	}
	
	public String getComment() {
		return comment;
	}
	public void setComment(String comment) {
		this.comment = comment;
	}

	//Working Constructor
	public Powersocket(String id, String ip, String password) {
		super(id);
		this.powerid = id;
		this.ip = ip;
		this.password = password;
		
		initHttpClient();
	}
	
	private void initHttpClient() {
	}
	
	//Functional
	public Boolean login() throws Exception {
		//System.out.println("=== Login ===");

		HttpClient httpclient = new DefaultHttpClient();
		HttpParams params = httpclient.getParams();
		HttpConnectionParams.setConnectionTimeout(params, 5000);
		HttpConnectionParams.setSoTimeout(params, 10000);
		
		try {
		    // Send data
		    String url = "http://" + this.ip + "/login.html";
		    HttpPost httpost = new HttpPost(url);
		    
		    //POST with login
		    List <NameValuePair> nvps = new ArrayList <NameValuePair>();
            nvps.add(new BasicNameValuePair("pw", this.password));
            httpost.setEntity(new UrlEncodedFormEntity(nvps, HTTP.UTF_8));

            HttpResponse response = httpclient.execute(httpost);
            HttpEntity entity = response.getEntity();

            if (response.getStatusLine().getStatusCode() != HttpStatus.SC_OK) {
            	throw new HttpException();
            }

            //Check if login response content contains socket states
            if (entity != null) {
            	String returnhtml =  EntityUtils.toString(entity);
            	if (getSocketStatesByHtml(returnhtml) != null)
            		return true;
            	else
            		return false;
            }
            EntityUtils.consume(entity);
           
		} catch (Exception e) {
            throw e;
		} finally {
			httpclient.getConnectionManager().shutdown();
		}
		
		return false;
	};
	
	public Boolean logout() throws Exception {	
		//System.out.println("=== Logout ===");

		HttpClient httpclient = new DefaultHttpClient();
		HttpParams params = httpclient.getParams();
		HttpConnectionParams.setConnectionTimeout(params, 5000);
		HttpConnectionParams.setSoTimeout(params, 10000);
		
		try {
		    String url = "http://" + this.ip + "/login.html";
		    HttpGet httpget = new HttpGet(url);
	        HttpResponse response = httpclient.execute(httpget);
	
	        if (response.getStatusLine().getStatusCode() != HttpStatus.SC_OK) {
	        	throw new HttpException();
	        } else {
	        	return true;
	        }
		} catch (HttpException e) {
			System.out.println("Exception in Logout.");
			//e.printStackTrace();
        	throw e;
		} catch (ClientProtocolException e) {
			e.printStackTrace();
		} catch (IOException e) {
			e.printStackTrace();
		} finally {
			httpclient.getConnectionManager().shutdown();
		}
    	return false;
	}
	
	public void updateSocketStates() throws Exception {
		this.login(); //will update socket array.
		this.logout();
	}
	
	/*
	 * Get updated Socketstates (including request to powersocket)
	 */
	public Boolean[] getSocketStates() {
		try {
			this.updateSocketStates();
		} catch (Exception e) {
			System.err.println("Catched Exception by getSocketstates.");
		}
		
		return socketStates;
	}
	
	/*
	 * Get updated Socketstates as String (including request to powersocket)
	 */
	public String getSocketString() {
		//System.out.println("=== getSocketString ===");
		try {
			this.updateSocketStates();
		} catch (Exception e) {
			System.err.println("Catched Exception by getSocketString.");
		}
		String socketstring = this.socketStates[0] + "," + this.socketStates[1] + "," + this.socketStates[2] + "," + this.socketStates[3];
		socketstring = socketstring.replace("true", "1");
		socketstring = socketstring.replace("false", "0");
		return socketstring;
	}
	
	private Boolean[] getSocketStatesByHtml(String html) {
		int ctlpos = html.indexOf("var ctl = [");
    	if (ctlpos >= 0) {
    		String socketctls = html.substring(ctlpos+11, ctlpos+11+7);
    		this.socketStates[0] = (socketctls.substring(0, 1).equals("1"));
    		this.socketStates[1] = (socketctls.substring(2, 3).equals("1"));
    		this.socketStates[2] = (socketctls.substring(4, 5).equals("1"));
    		this.socketStates[3] = (socketctls.substring(6, 7).equals("1"));

    		socketctls = null;
    		
    		return this.socketStates;
    	} else {
    		return null;
    	}
	}
	
	
	public Boolean switchsocket(Integer socketnumber, Boolean newstate) throws Exception {
		try {
			this.login();
		} catch (HttpException e) {
			throw new PowersocketUnreachableException();
		}
			
		Integer socket = socketnumber - 1;
		//System.out.println("=== Powersocket #"+this.getId() +" switch " +socket+ " to "+newstate+" ===");
		
		//Already in state?
		if (this.socketStates[socket] == newstate) {
			System.out.println("Not switched. Was already in target state");
			return true;
		}

		DefaultHttpClient httpclient = new DefaultHttpClient();
		HttpParams params = httpclient.getParams();
		HttpConnectionParams.setConnectionTimeout(params, 5000);
		HttpConnectionParams.setSoTimeout(params, 10000);
		//Does not work... commented out.
//		
//		HttpRequestRetryHandler myRetryHandler = new HttpRequestRetryHandler() {
//
//		    public boolean retryRequest(
//		            IOException exception, 
//		            int executionCount,
//		            HttpContext context) {
//		        if (executionCount >= 1) {
//		            return false;
//		        }
//		        if (exception instanceof NoHttpResponseException) {
//		            // Retry if the server dropped connection on us
//		            return false;
//		        }
//		        if (exception instanceof SSLHandshakeException) {
//		            // Do not retry on SSL handshake exception
//		            return false;
//		        }
//		        HttpRequest request = (HttpRequest) context.getAttribute(
//		                ExecutionContext.HTTP_REQUEST);
//		        boolean idempotent = !(request instanceof HttpEntityEnclosingRequest); 
//		        if (idempotent) {
//		            // Retry if the request is considered idempotent 
//		            return false;
//		        }
//		        return false;
//		    }
//
//		};
//		httpclient.setHttpRequestRetryHandler(myRetryHandler);
		
		try {
		    String url = "http://" + this.ip + "/";
		    HttpPost httppost = new HttpPost(url);
		    
		    //POST with login
		    List <NameValuePair> nvps = new ArrayList <NameValuePair>();
		    Integer newstateint = newstate ? 1 : 0;
            nvps.add(new BasicNameValuePair("ctl"+socketnumber, newstateint.toString()));
            httppost.setEntity(new UrlEncodedFormEntity(nvps, HTTP.UTF_8));

            HttpResponse response = httpclient.execute(httppost);
            HttpEntity entity = response.getEntity();

            if (response.getStatusLine().getStatusCode() != HttpStatus.SC_OK) {
            	throw new HttpException();
	        } else {
	        	//Check if login response content contains correct content
	            if (entity != null) {
	            	String returnhtml =  EntityUtils.toString(entity);
	                System.out.println(returnhtml);
	            	Boolean[] newSocketStates = getSocketStatesByHtml(returnhtml);	
	            	
	            	//System.out.println("NewSocket: "+  newSocketStates[socket] + " should be: "+newstate);
	            	//System.out.println("NewSocket Length: " + newSocketStates.length);
	            	
	            	if (newSocketStates[socket] != null 
	            			&& newSocketStates[socket] == newstate) {
	            		System.out.println("Powersocket #"+getId() + " switched " + socket + "to " + (newstate ? "on" : "off"));
	            	} else {
	            		throw new Exception("Socket not switched. Switch action failed.");
	            	}
	            }
	            EntityUtils.consume(entity);
	        }
		} catch (SocketException e) {
			System.out.println("Powersocket not available. (Network no answer)");
		} catch (Exception e) {
			System.out.println("Exception in switch socket.");
			throw e;
		} finally {
			httpclient.getConnectionManager().shutdown();
			logout();
		}
		return true;
	}
	
	public Boolean fireAction(List<String> actions) {
		if (actions.size() == 2) {
			try {
				//Switch every parseInt of onArray (at 0) to on
				String[] onArray = actions.get(0).split(",");
				for (int i = 0; i < onArray.length; i++) {
					String string = onArray[i];
					if (string != null && !string.equals(""))
						this.switchsocket(Integer.parseInt(string), true);
				}

				//Switch every parseInt of offArray (at 1) to off
				String[] offArray = actions.get(1).split(",");
				for (int i = 0; i < offArray.length; i++) {
					String string = offArray[i];
					this.switchsocket(Integer.parseInt(string), false);

				}
	
				return true;
			} catch (PowersocketUnreachableException e) {
				System.err.println("Powersocket #"+ getId() +" is unreachable.");
				return false;
			} catch (Exception e) {
				System.err.println("Parse Error to switch Socket in ActionString: "+e.getLocalizedMessage());
				e.printStackTrace();
				return false;
			}
		} else {
			System.err.println("Invalid ActionString to fire Action. Should length = 2.");
			return false;
		}
	}
	
	public String toString() {
		return "Powersocket '" + this.powerid + "' " + this.ip + " with '" + this.password + "'";
	}
	
	public String toJSON() {
		StringBuilder sb = new StringBuilder();
        sb.append("{\n");
        sb.append("\"id\": \"" + this.getPowerid() + "\",\n");
        sb.append("\"ip\": \""+ this.getIp() + "\",\n");
        sb.append("\"password\": \""+ this.getPassword() + "\",\n");
        sb.append("\"states\": [" + this.getSocketString() +"],\n");
        sb.append("\"comment\": \"" + this.getComment() + "\"\n");
        sb.append("}");
        return sb.toString();
	}
	
}
