package edu.dhbw.inputs;
import java.util.Date;
import java.util.Set;
import java.util.SortedMap;
import java.util.TreeMap;

public class Sensor extends BaseInput {
	private String location;
	private String measure;
	private Integer flapzone;
	
	protected SortedMap<Date, Double> history;
	protected final static Integer HISTORY_LIMIT = 10;

	public String getLocation() {
		return location;
	}

	public void setLocation(String location) {
		this.location = location;
	}

	public String getMeasure() {
		return measure;
	}

	public void setMeasure(String measure) {
		this.measure = measure;
	}
	
	@Override
	public String getInput() {
		if (getValue() != null)
			return getValue().toString();
		else
			return null;
	}

	@Override
	public void setInput(String input) {
		setValue(Double.parseDouble(input));
	}
		
	public Double getValue() {
		if (history != null && history.size() > 0)
			return history.get(history.lastKey());
		else
			return null;
	}

	public void setValue(Double value) {
		//Check if limit size, then remove one
		if (this.history.size() == HISTORY_LIMIT) {
			this.history.remove(this.history.firstKey());
		}
		//Add new value to history
		Date now = new Date();
		this.history.put(now, value);
	}

	public SortedMap<Date, Double> getHistory() {
		return history;
	}

	public Integer getFlapzone() {
		return flapzone;
	}

	public void setFlapzone(Integer flapzone) {
		this.flapzone = flapzone;
	}
	
	//Working Constructor
	public Sensor(String sensorid) {
		super(sensorid);
		this.history = new TreeMap<Date, Double>();
	}
	
	public String toString() {
		StringBuilder sb = new StringBuilder();
		sb.append("Sensor #"+ getId() + " = " + getValue() + "@"+ getLocation() +"\n");
		return sb.toString();
	}
	
	public String toJSON() {
		StringBuilder json = new StringBuilder();
		json.append("{\n");
			json.append("\t\"id\": \"" + getId() + "\",\n");
			json.append("\t\"input\": \"" + getInput() + "\",\n");
			json.append("\t\"type\": \"" + getType() + "\",\n");
			json.append("\t\"history\": [\n");
			
			Set<Date> pastValues = history.keySet();
			for (Date date : pastValues) {
				json.append("\t\t{\"date\": \""+date.toString()+"\", \"value\": "+history.get(date)+"}");
				if (!history.lastKey().equals(date))
					json.append(",\n");
			}
			json.append("],\n");
			json.append("\t\"comment\": \"" + getComment() + "\"\n");

		json.append("}");
		return json.toString();
	}
}
