package edu.dhbw.resources;
import java.io.IOException;
import java.util.ArrayList;
import java.util.HashMap;
import java.util.Iterator;
import java.util.List;

import org.json.JSONArray;
import org.json.JSONException;
import org.json.JSONObject;
import org.restlet.data.MediaType;
import org.restlet.data.Status;
import org.restlet.representation.Representation;
import org.restlet.representation.StringRepresentation;
import org.restlet.resource.Get;
import org.restlet.resource.Post;
import org.restlet.resource.ResourceException;

import edu.dhbw.brainbox.BrainApplication;
import edu.dhbw.brainbox.outputs.Powersocket;


public class PowersocketResource extends BrainBaseResource {
	private Powersocket powersocket;
	private String powerid;
	
	@Override
    public void doInit() {
		powerid = (String) getRequestAttributes().get("powerid");
        this.powersocket = findPowersocket(powerid);
    }
	
	private Powersocket findPowersocket(String powerid) {
		return ((BrainApplication) getApplication()).getPowersocketById(powerid);
	}
	
	private Boolean savePowersocket(Powersocket powersocket) {
		((BrainApplication) getApplication()).putPowersocket(powersocket);
		return true;
	}
	
	private void updatePowersocket(String id, String ip, String password, String comment) {
		Powersocket power = findPowersocket(id);
		if (power == null && id != null && ip != null && password != null)
			power = new Powersocket(id, ip, password);
		
		if (ip != null)
			power.setIp(ip);
		if (password != null)
			power.setPassword(password);
		if (comment != null)
			power.setComment(comment);
		
		savePowersocket(power);
	} 
    
    /** 
     * Handle POST requests: Receive an Action on Items (switch Powersocket)
     */  
    @Post("json")
    public Representation jsonactionReceived(Representation entity) {  
    	System.out.println("=== JSON POST to Powersocket #"+powerid+" ===");
    	
    	try {
			JSONObject json = new JSONObject(entity.getText());
			
			JSONArray jsonOn = json.has("on") ? json.getJSONArray("on") : null;
			JSONArray jsonOff = json.has("off") ? json.getJSONArray("off") : null;
			
			if (jsonOn == null || jsonOff == null) {
				String ip = json.has("ip") ? json.getString("ip") : null;
				String pw = json.has("password") ? json.getString("password") : null;
				String comment = json.has("comment") ? json.getString("comment") : null;
				
				updatePowersocket(json.getString("id"), ip, pw, comment);
				
	            setStatus(Status.SUCCESS_OK);  
	            Representation rep = new StringRepresentation("{\"result\": \"success\"}",  
	                    MediaType.TEXT_PLAIN);  
	            return rep;
				
			} else {
				
				ArrayList<Integer> onList = new ArrayList<Integer>();
				for (int i=0; i<jsonOn.length(); i++) {
					onList.add( jsonOn.getInt(i) );
				}			
				ArrayList<Integer> offList = new ArrayList<Integer>();
				for (int i=0; i<jsonOff.length(); i++) {
					offList.add( jsonOff.getInt(i) );
				}
				
		        HashMap<String, ArrayList<Integer>> actionList = new HashMap<String, ArrayList<Integer>>();
		        actionList.put("on", onList);
		        actionList.put("off", offList);
		        
		        if (this.processSwitchActions(actionList)) {
		        	// Set the response's status and entity  
		            setStatus(Status.SUCCESS_OK);  
		            Representation rep = new StringRepresentation("{\"result\": \"success\"}",  
		                    MediaType.TEXT_PLAIN);  
		            return rep;
		        } else {
		            setStatus(Status.SUCCESS_OK);  
		            Representation rep = new StringRepresentation("{\"result\": \"error\", \"more\": \"Switch Actions not processed. (JSON)\"}",  
		                    MediaType.TEXT_PLAIN);  
		            return rep;
		        }
			}
		} catch (JSONException e) {
			e.printStackTrace();
		} catch (IOException e) {
			e.printStackTrace();
		}
    	
		setStatus(Status.CLIENT_ERROR_UNPROCESSABLE_ENTITY);  
        Representation rep = new StringRepresentation("{\"result\": \"error\", \"more\": \"Switch Actions not processed. (JSON)\"}",  
                MediaType.TEXT_PLAIN);  
        return rep;
    }  
    
    private Boolean processSwitchActions(HashMap<String, ArrayList<Integer>> actionarray) {
    	try {
	    	ArrayList<Integer> onList = actionarray.get("on");
	    	for (Integer socket : onList) {
				powersocket.switchsocket(socket, true);
			}
	    	
	    	ArrayList<Integer> offList = actionarray.get("off");
	    	for (Integer socket : offList) {
				powersocket.switchsocket(socket, false);
			}
	    	return true;
	    	
    	} catch (Exception e) {
    		return false;
    	}
    }

	/** 
     * Handle GET requests: Print out the StringRepresentation of Item (Powersocket)
     */  
	@Get("json")
    public Representation printPowersocketByJSON(Representation entity) throws ResourceException {
        Representation result = null;

        System.out.println("GET JSON Powersocket " + powerid + (this.powersocket == null ? "null" : this.powersocket.toString()));
        
        final StringBuilder sb = new StringBuilder();
        
        if (this.powersocket != null) {
            // Creates a text representation
            result = new StringRepresentation(powersocket.toJSON());
        } else {
        	sb.append("[");
        	List<Powersocket> powersockets = getContainer().queryByExample(new Powersocket(null, null, null));
        	for (Iterator<Powersocket> iterator = powersockets.iterator(); iterator.hasNext();) {
				Powersocket powersocket = (Powersocket) iterator.next();
				sb.append('"'+powersocket.getPowerid()+'"');
				if (iterator.hasNext())
					sb.append(",");
			}
        	sb.append("]");
            result = new StringRepresentation(sb, MediaType.TEXT_JAVASCRIPT);
            
        }
        
        return result;
    }
	
//	@Get("xml|html")
//    public Representation printPowersocket(Representation entity) throws ResourceException {
//        Representation result = null;
//
//        System.out.println("GET HTML Powersocket " + (this.powersocket != null ? this.powersocket.getId() : "null"));
//        
//        final StringBuilder sb = new StringBuilder();
//        
//        if (this.powersocket != null) {
//            // Creates a text representation
//            sb.append("\"id\": \"" + powersocket.getPowerid() + "\",\n");
//            sb.append("\"ip\": \""+ powersocket.getIp() + "\",\n");
//            sb.append("\"password\": \""+ powersocket.getPassword() + "\",\n");
//            sb.append("\"states\": [" + this.powersocket.getSocketString() +"],\n");
//            sb.append("\"comment\": \"" + this.powersocket.getComment() + "\"\n");
//            result = new StringRepresentation(sb);
//        } else {
//        	sb.append("<html><body>");
//        	sb.append("No Powersocket with " + this.powerid + " found :-(<br/>");
//        	List<Powersocket> powersockets = getContainer().queryByExample(new Powersocket(null, null, null));
//        	for (Powersocket powersocket : powersockets) {
//				sb.append("<a href=\""+powersocket.getPowerid()+"\"/>"+powersocket.getPowerid()+"</a><br/>");
//			}
//        	sb.append("</body></html>");
//            result = new StringRepresentation(sb, MediaType.TEXT_HTML);
//            
//        }
//        
//        return result;
//    }
	
	
//	/** 
//     * Handle POST requests: Receive an Action on Items (switch Powersocket)
//     */  
//    @Post("xml|html")
//    public Representation actionReceived(Representation entity) {  
//        
//        //Get POST Vars
//        Form form = new Form(entity);  
//        if (form.getFirstValue("on") == null
//        		&& form.getFirstValue("off") == null) {
//        	
//        	// Set the response's status and entity  
//            setStatus(Status.SUCCESS_OK);  
//            Representation repNothing = new StringRepresentation("Nothing to do. No switch actions found.", 
//                    MediaType.TEXT_PLAIN);  
//            
//            // Indicates where is located the new resource.  
//            //rep.setIdentifier(getRequest().getResourceRef().getIdentifier());  
//            return repNothing;
//        }
//        
//        String[] actionOn = {};
//        String[] actionOff = {};
//        if (form.getFirstValue("on") != null) 
//        	actionOn = form.getFirstValue("on").split(",");  
//        if (form.getFirstValue("off") != null)
//        	actionOff = form.getFirstValue("off").split(",");
//        
//        ArrayList<Integer> onList = new ArrayList<Integer>();
//        for (String action : actionOn) {
//        	onList.add(Integer.parseInt(action));
//		}
//        
//        ArrayList<Integer> offList = new ArrayList<Integer>();
//        for (String action : actionOff) {
//        	offList.add(Integer.parseInt(action));
//		}
//        
//        HashMap<String, ArrayList<Integer>> actionList = new HashMap<String, ArrayList<Integer>>();
//        actionList.put("on", onList);
//        actionList.put("off", offList);
//        
//        if (this.processSwitchActions(actionList)) {
//        	// Set the response's status and entity  
//            setStatus(Status.SUCCESS_OK);  
//            Representation rep = new StringRepresentation("{\"result\": \"success\"}",  
//                    MediaType.TEXT_PLAIN);  
//            return rep;
//        } else {
//            setStatus(Status.SUCCESS_OK);  
//            Representation rep = new StringRepresentation("{\"result\": \"error\", \"more\": \"Switch Actions not processed. (Form)\"}",  
//                    MediaType.TEXT_PLAIN);  
//            return rep;
//        }
//    } 
}
