package edu.dhbw.resources;
import java.io.IOException;
import java.util.Iterator;
import java.util.List;

import org.json.JSONException;
import org.json.JSONObject;
import org.restlet.data.MediaType;
import org.restlet.data.Status;
import org.restlet.representation.Representation;
import org.restlet.representation.StringRepresentation;
import org.restlet.resource.Get;
import org.restlet.resource.Post;
import org.restlet.resource.ResourceException;

import edu.dhbw.brainbox.BrainApplication;
import edu.dhbw.inputs.Sensor;


public class SensorResource extends BrainBaseResource {
	private Sensor sensor;
	private String sensorid;

	@Override
    public void doInit() {
        sensorid = (String) getRequestAttributes().get("sensorid");
        this.sensor = findSensor(sensorid);
	}
	
	private Sensor findSensor(String sensorid) {
		return ((BrainApplication) getApplication()).getSensorById(sensorid);
	}
	private Boolean saveSensor(Sensor sensor, Boolean valueChanged) {
		((BrainApplication) getApplication()).saveSensor(sensor, valueChanged);
		return true;
	}
	
	private void updateSensor(String id, Double newvalue, String comment, String location, String type) {
        Sensor updateSensor = findSensor(id);
        //Change existing and save it.
        if (updateSensor == null) {
        	updateSensor = new Sensor(id);
        }
        
        Boolean valueChanged = false;
        if (newvalue != null) {
        	updateSensor.setValue(newvalue);
        	valueChanged = true;
        }
        if (comment != null)
        	updateSensor.setComment(comment);
        if (location != null)
        	updateSensor.setLocation(location);
        if (type != null) 
        	updateSensor.setType(type);
        
        //save
        this.saveSensor(updateSensor, valueChanged);
	}
    
    /** 
     * Handle json POST requests: Receive new value for sensor
     * @throws IOException 
     */  
    @Post("json")
    public Representation jsonactionReceived(Representation entity) {  
    	System.out.println("=== JSON POST to Sensor ===");
    	
    	try {
			JSONObject json = new JSONObject(entity.getText().toString());
			Double newvalue = json.has("value") ? json.getDouble("value") : null;
			String comment = json.has("comment") ? json.getString("comment") : null;
			String location = json.has("location") ? json.getString("location") : null;
			String type = json.has("type") ? json.getString("type") : null;
			
			updateSensor(json.getString("id"), newvalue, comment, location, type); //use context id?
	        
        	// Set the response's status and entity  
            setStatus(Status.SUCCESS_OK);  
            Representation rep = new StringRepresentation("{\"result\": \"success\"}",  
                    MediaType.TEXT_PLAIN);  
            return rep;

		} catch (JSONException e) {
			e.printStackTrace();
		} catch (IOException e) {
			e.printStackTrace();
		}
		
		setStatus(Status.CLIENT_ERROR_UNPROCESSABLE_ENTITY);  
        Representation rep = new StringRepresentation("{\"result\": \"error\", \"more\": \"Failed to parse JSON.\"}",  
                MediaType.TEXT_PLAIN);  
        return rep;
    }  
    
	/** 
     * Handle GET requests: Print out the StringRepresentation of Item (Powersocket)
     */  
	@Get("json")
    public Representation printPowersocketAsJSON(Representation entity) throws ResourceException {
        Representation result = null;

        System.out.println("GET JSON Sensor #"+this.sensorid+ " = " + (this.sensor == null ? "null" : this.sensor.toString()));
        
        if (this.sensor != null) {
            // Creates a text representation
            result = new StringRepresentation(this.sensor.toJSON());
        } else {
        	StringBuilder sb = new StringBuilder();
        	sb.append("[");
        	List<Sensor> sensors = getContainer().queryByExample(new Sensor(null));
        	for (Iterator<Sensor> iterator = sensors.iterator(); iterator.hasNext();) {
				Sensor sensor = (Sensor) iterator.next();
				sb.append("\""+sensor.getId()+"\"");
				if (iterator.hasNext())
					sb.append(",");
			}
        	sb.append("]");
            result = new StringRepresentation(sb, MediaType.TEXT_JAVASCRIPT);
        }
        
        return result;
    }

//	/** 
//     * Handle GET requests: Print out the StringRepresentation of Item
//     */  
//	@Get("xml|html")
//    public Representation printPowersocket(Representation entity) throws ResourceException {
//        Representation result = null;
//
//        System.out.println("GET Sensor (xml|html)");
//        
//        if (this.sensor != null) {
//            // Creates a text representation
//            result = new StringRepresentation(this.sensor.toJSON());
//        } else {
//        	StringBuilder sb = new StringBuilder();
//        	sb.append("<html><body>");
//        	sb.append("No Sensor #" + this.sensorid + " found :-(<br/>");
//        	List<Sensor> sensors = getContainer().queryByExample(new Sensor(null));
//        	for (Sensor sensor : sensors) {
//				sb.append("<a href=\""+sensor.getId()+"\"/>"+sensor.getId()+"</a><br/>");
//			}
//        	sb.append("</body></html>");
//            result = new StringRepresentation(sb, MediaType.TEXT_HTML);
//        }
//        
//        return result;
//    }
}
