package edu.dhbw.resources;
import java.io.IOException;
import java.util.ArrayList;
import java.util.Iterator;
import java.util.List;

import org.json.JSONArray;
import org.json.JSONException;
import org.json.JSONObject;
import org.restlet.data.MediaType;
import org.restlet.data.Status;
import org.restlet.representation.Representation;
import org.restlet.representation.StringRepresentation;
import org.restlet.resource.Get;
import org.restlet.resource.Post;
import org.restlet.resource.ResourceException;

import edu.dhbw.brainbox.BrainApplication;
import edu.dhbw.brainbox.outputs.AktorBoxcarAPI;
import edu.dhbw.brainbox.outputs.BaseAktor;
import edu.dhbw.brainbox.outputs.Powersocket;
import edu.dhbw.inputs.BaseInput;
import edu.dhbw.inputs.Sensor;
import edu.dhbw.triggers.BaseTrigger;
import edu.dhbw.triggers.TriggerEvent;


public class TriggerEventResource extends BrainBaseResource {
	private BaseTrigger trigger;
	private String triggerid;

	@Override
    public void doInit() {
        triggerid = (String) getRequestAttributes().get("triggerid");
        this.trigger = findTrigger(triggerid);
        System.out.println("GET Trigger "+triggerid + " " + (this.trigger != null ? this.trigger.toString() : "null"));
        //FIXME: Trigger is not null, even if no triggerid is in context available. (see GET Output in sysout)
	}
	
	private BaseTrigger findTrigger(String triggerid) {
		return ((BrainApplication) getApplication()).getTriggerById(triggerid);
	}
	private BaseAktor findAktor(String id) {
		return ((BrainApplication) getApplication()).getPowersocketById(id);
	}
	private BaseInput findSensor(String id) {
		return ((BrainApplication) getApplication()).getSensorById(id);
	}
	private Boolean saveTrigger(BaseTrigger trigger) {
		((BrainApplication) getApplication()).saveTrigger(trigger);
		return true;
	}
	
	private void updateTrigger(String id, String sensorid, String aktorid, Double flapping, Double limit, Integer direction, List<String> actionstring, String desc) {
        TriggerEvent updateTrigger = (TriggerEvent) findTrigger(id);
        
        Sensor sensor = (Sensor) findSensor(sensorid);
        if (sensor == null)
        	System.out.println("Sensor #"+sensorid+" not found.");
        else
        	System.out.println("Sensor #"+sensor.getId()+" found.");
        
        //TODO: replace switch workaround at aktor with generic architecture...
        BaseAktor aktor;
        if (aktorid.equals("boxcar")) {
        	aktor = new AktorBoxcarAPI("boxcar", actionstring.get(0)); //TODO: make message changeable...
        	System.out.println("created boxcar aktor for this trigger: #"+aktor.getId());
        } else {
	        aktor = (Powersocket) findAktor(aktorid);
	        if (aktor == null)
	        	System.out.println("Aktor #"+aktorid+" not found.");
	        else
	        	System.out.println("Aktor #"+aktor.getId()+" found.");
        }
        
        //Change existing and save it.
        if (updateTrigger == null) {
        	updateTrigger = new TriggerEvent(id, null, null);
        }
        
        if (sensorid != null) {
        	updateTrigger.setSensor(sensor);
        }
        if (aktorid != null) {
        	updateTrigger.setAktor(aktor);
        }
        if (flapping != null)
        	updateTrigger.setFlapping(flapping);
        if (limit != null) 
        	updateTrigger.setLimit(limit);
        if (direction != null) {
        	updateTrigger.setDirection(direction);
        }
        if (actionstring != null) {
        	updateTrigger.setActionstring(actionstring);
        }
        if (desc != null)
        	updateTrigger.setDescription(desc);
        
        //save
        this.saveTrigger(updateTrigger);
	}
	
    /** 
     * Handle json POST requests: Receive new value for sensor
     * @throws IOException 
     */  
    @Post("json")
    public Representation jsonactionReceived(Representation entity) {  
    	System.out.println("=== JSON POST to TriggerEvent ===");
    	
    	try {
			JSONObject json = new JSONObject(entity.getText().toString());
			String sensorid = json.has("sensorid") ? json.getString("sensorid") : null;
			String aktorid = json.has("aktorid") ? json.getString("aktorid") : null;
			Double flapping = json.has("flapping") ? json.getDouble("flapping") : null;
			Double limit = json.has("limit") ? json.getDouble("limit") : null;
			Integer direction = json.has("direction") ? json.getInt("direction") : null;
			String description = json.has("description") ? json.getString("description") : null;
			
			List<String> actionlist = new ArrayList<String>();
			if (json.has("actionstring")) {
				JSONArray actions = json.getJSONArray("actionstring");
				for (int i=0; i<actions.length(); i++) {
					actionlist.add( actions.getString(i) );
				}	
			} else {
				actionlist = null;
			}

			updateTrigger(json.getString("id"), sensorid, aktorid, flapping, limit, direction, actionlist, description); //use context id?
	        
        	// Set the response's status and entity  
            setStatus(Status.SUCCESS_OK);  
            Representation rep = new StringRepresentation("{\"result\": \"success\"}",  
                    MediaType.TEXT_PLAIN);  
            return rep;

		} catch (JSONException e) {
			e.printStackTrace();
		} catch (IOException e) {
			e.printStackTrace();
		}
		
		setStatus(Status.CLIENT_ERROR_UNPROCESSABLE_ENTITY);  
        Representation rep = new StringRepresentation("{\"result\": \"error\", \"more\": \"Failed to parse JSON.\"}",  
                MediaType.TEXT_PLAIN);  
        return rep;
    }  
	
	/** 
     * Handle GET requests: Print out the StringRepresentation of Item (Powersocket)
     */  
	@Get("json")
    public Representation printTriggerAsJSON(Representation entity) throws ResourceException {
        Representation result = null;

        System.out.println("GET TriggerEvent JSON");
        
        if (triggerid != null) {
            // Creates a text representation
            result = new StringRepresentation(this.trigger.toJSON());
        } else {

    		// Query the database and get the first result
    		final List<BaseTrigger> triggers = getContainer().queryByExample(new BaseTrigger(null));
    		
        	StringBuilder sb = new StringBuilder();
        	sb.append("[");
        	for (Iterator<BaseTrigger> iterator = triggers.iterator(); iterator.hasNext();) {
        		BaseTrigger trigger = (BaseTrigger) iterator.next();
				sb.append("\""+trigger.getId()+"\"");
				if (iterator.hasNext())
					sb.append(",");
			}
        	sb.append("]");
            result = new StringRepresentation(sb, MediaType.TEXT_JAVASCRIPT);
        }
        
        return result;
    }
}
